@@ -7,6 +7,9 @@ Created on Sat Apr 30 23:04:13 2022
 NN models and their prediction with numpy based calculations
 Keras and tensorflow gets locked for prediction in Multi-thread mode
 
+Numpy version of prediction for both DNN and CNN models are implemented, but bit slow
+but we can use multiprocessing
+
 """
 import numpy as np
 import os
@@ -156,14 +159,16 @@ def model_arch_CNN_DNN_optimized(shape,
             conv1 = keras.layers.Conv1D(filters=CNN_filters[lay], kernel_size=kernel_size[lay], 
                                         strides=stride[lay], 
                                         activation=layer_activation, name="Conv_"+str(lay+1))(inputs)
-            pool1 = keras.layers.MaxPooling1D(pool_size=pool_size[lay], \
-                                              name="Pool_"+str(lay+1))(conv1)
+            if pool_size[lay] != 1 or pool_size[lay] != 0:
+                pool1 = keras.layers.MaxPooling1D(pool_size=pool_size[lay], \
+                                                  name="Pool_"+str(lay+1))(conv1)
         else:
             conv1 = keras.layers.Conv1D(filters=CNN_filters[lay], kernel_size=kernel_size[lay], 
                                         strides=stride[lay], 
                                         activation=layer_activation, name="Conv_"+str(lay+1))(pool1)
-            pool1 = keras.layers.MaxPooling1D(pool_size=pool_size[lay], \
-                                              name="Pool_"+str(lay+1))(conv1)
+            if pool_size[lay] != 1 or pool_size[lay] != 0:
+                pool1 = keras.layers.MaxPooling1D(pool_size=pool_size[lay], \
+                                                  name="Pool_"+str(lay+1))(conv1)
     flatten = keras.layers.Flatten(name="Flatten")(pool1)
 
     for lay in range(DNN_layers):
@@ -322,18 +327,6 @@ def predict_DNN_onelayer(x, wb, temp_key):
     layer3 = softmax(layer1) ## output softmax activation
     return layer3
 
-def predict_CNN_DNN(x, wb, temp_key):
-    json_file = open(r"C:\Users\purushot\Desktop\github_version_simple\lauetoolsnn\multi_mat_LaueNN\Zr_alpha_ZrO2_mono\model_Zr_alpha_ZrO2_mono.json", 'r')
-    load_weights = r"C:\Users\purushot\Desktop\github_version_simple\lauetoolsnn\multi_mat_LaueNN\Zr_alpha_ZrO2_mono\model_Zr_alpha_ZrO2_mono.h5"
-    # # load json and create model
-    loaded_model_json = json_file.read()
-    json_file.close()
-    model = model_from_json(loaded_model_json)
-    model.load_weights(load_weights)
-    # np.savez_compressed(r"C:\Users\purushot\Desktop\github_version_simple\lauetoolsnn\multi_mat_LaueNN\Zr_alpha_ZrO2_mono\model_Zr_alpha_ZrO2_mono.npz", x, prediction)
-    return model.predict(x)
-
-
 def predict_with_file(x, model_direc=None, material_=None):
     if model_direc!=None and material_!=None:
         if len(material_) > 1:
@@ -354,9 +347,179 @@ def predict_with_file(x, model_direc=None, material_=None):
         model.load_weights(load_weights)
         return model.predict(x)
 
-if __name__ == "__main__":
+# =============================================================================
+# 1D CNN numpy implementation
+# =============================================================================
+    
+def asStride(arr, sub_shape, stride):
+    '''Get a strided sub-matrices view of an ndarray.
+    Args:
+        arr (ndarray): input array of rank 4, with shape (m, hi, wi, ci).
+        sub_shape (tuple): window size: (f1, f2).
+        stride (int): stride of windows in both 2nd and 3rd dimensions.
+    Returns:
+        subs (view): strided window view.
+    This is used to facilitate a vectorized 3d convolution.
+    The input array <arr> has shape (m, hi, wi, ci), and is transformed
+    to a strided view with shape (m, ho, wo, f, f, ci). where:
+        m: number of records.
+        hi, wi: height and width of input image.
+        ci: channels of input image.
+        f: kernel size.
+    The convolution kernel has shape (f, f, ci, co).
+    Then the vectorized 3d convolution can be achieved using either an einsum()
+    or a tensordot():
+        conv = np.einsum('myxfgc,fgcz->myxz', arr_view, kernel)
+        conv = np.tensordot(arr_view, kernel, axes=([3, 4, 5], [0, 1, 2]))
+    See also skimage.util.shape.view_as_windows()
+    '''
+    if len(arr.shape) == 2:
+        m, hi  = arr.shape
+        ci = 1
+        sm, sh = arr.strides
+        sc = 1
+    else:
+        m, hi, ci  = arr.shape
+        sm, sh, sc = arr.strides    
+    view_shape = (m, 1+(hi-sub_shape)//stride, sub_shape, ci)
+    strides = (sm, stride*sh, sh, sc)
+    subs = np.lib.stride_tricks.as_strided(arr, view_shape, strides=strides, writeable=False)
+    return subs
+
+def calculate_output_dims(input_dims, _stride, _w, _padding):
+    """
+    :param input_dims - 4 element tuple (n, h_in, w_in, c)
+    :output 4 element tuple (n, h_out, w_out, n_f)
+    ------------------------------------------------------------------------
+    n - number of examples in batch
+    w_in - width of input volume
+    h_in - width of input volume
+    w_out - width of input volume
+    h_out - width of input volume
+    c - number of channels of the input volume
+    n_f - number of filters in filter volume
+    """
+    if len(input_dims) == 2:
+        n, h_in = input_dims
+    elif len(input_dims) == 3:
+        n, h_in, _ = input_dims
+    else:
+        print("unsupported input shape")
+        
+    h_f, w_f, n_f = _w.shape
+    if _padding == 'same':
+        return n, h_in, n_f
+    elif _padding == 'valid':
+        h_out = (h_in - h_f) // _stride + 1
+        return n, h_out, n_f
+    else:
+        print(f"Unsupported padding value: {_padding}")
+
+def ReluLayer_forward_pass(a_prev):
+    """
+    :param a_prev - ND tensor with shape (n, ..., channels)
+    :output ND tensor with shape (n, ..., channels)
+    ------------------------------------------------------------------------
+    n - number of examples in batch
+    """
+    _z = np.maximum(0, a_prev)
+    return _z
+
+def softmax_forward_pass(a_prev):
+    """
+    :param a_prev - 2D tensor with shape (n, k)
+    :output 2D tensor with shape (n, k)
+    ------------------------------------------------------------------------
+    n - number of examples in batch
+    k - number of classes
+    (np.exp(x).T / np.sum(np.exp(x).T, axis=0)).T
+    """
+    e = np.exp(a_prev - a_prev.max(axis=1, keepdims=True))
+    _z = e / np.sum(e, axis=1, keepdims=True)
+    return _z
+
+def flatten_forward_pass(a_prev):
+    """
+    :param a_prev - ND tensor with shape (n, ..., channels)
+    :output - 1D tensor with shape (n, 1)
+    ------------------------------------------------------------------------
+    n - number of examples in batch
+    """
+    return np.ravel(a_prev).reshape(a_prev.shape[0], -1)
+
+def pooling_forward_pass(a_prev, _stride, _pool_size):
+    """
+    :param a_prev - 4D tensor with shape(n, h_in, w_in, c)
+    :output 4D tensor with shape(n, h_out, w_out, c)
+    ------------------------------------------------------------------------
+    n - number of examples in batch
+    w_in - width of input volume
+    h_in - width of input volume
+    c - number of channels of the input/output volume
+    w_out - width of output volume
+    h_out - width of output volume
+    """
+    subs = asStride(a_prev, _pool_size, _stride)
+    output = subs.max(axis=2)
+    return output
+
+def forward_pass_DNN(a_prev, _w, _b):
+    """
+    :param a_prev - 2D tensor with shape (n, units_prev)
+    :output - 2D tensor with shape (n, units_curr)
+    ------------------------------------------------------------------------
+    n - number of examples in batch
+    units_prev - number of units in previous layer
+    units_curr -  number of units in current layer    
+    """
+    return np.dot(a_prev, _w) + _b
+
+def forward_pass_CNN(a_prev, _w, _b, _stride):
+    """
+    :param a_prev - 4D tensor with shape (n, h_in, w_in, c)
+    :output 4D tensor with shape (n, h_out, w_out, n_f)
+    ------------------------------------------------------------------------
+    n - number of examples in batch
+    w_in - width of input volume
+    h_in - width of input volume
+    w_out - width of input volume
+    h_out - width of input volume
+    c - number of channels of the input volume
+    n_f - number of filters in filter volume
+    """
+    output_shape = calculate_output_dims(a_prev.shape, _stride, _w, 'valid')        
+    _, h_out, _ = output_shape
+    h_f, _, _ = _w.shape
+    subs = asStride(a_prev, h_f, _stride)
+    # output = np.einsum("ijkl,klm->ijm", subs, _w) + _b
+    output = np.tensordot(subs, _w, axes=([2, 3], [0, 1])) + _b
+    return output
+
+
+def predict_CNN_DNN(x, wb, temp_key):
+    """
+    2 CNN layers[128,128] (stride[5,2], kernel[10,3] + maxpooling[2,1]) + Relu activation
+    flatten
+    Outpur DNN layer (softmax activation)
+    
+    """
+    x = forward_pass_CNN(x, wb[temp_key[1]], wb[temp_key[0]], 5)
+    x = ReluLayer_forward_pass(x)
+    x = pooling_forward_pass(x, 2, 2)
+    
+    x = forward_pass_CNN(x, wb[temp_key[3]], wb[temp_key[2]], 2)
+    x = ReluLayer_forward_pass(x)
+    x = pooling_forward_pass(x, 1, 1)
+    
+    x = flatten_forward_pass(x)
+    
+    x = forward_pass_DNN(x, wb[temp_key[5]], wb[temp_key[4]])
+    x = softmax_forward_pass(x)
+    return x
+
+# if __name__ == "__main__":
+#     pass
     ## test of numpy prediction
-    pass
     # codebar = np.load(r"C:\Users\purushot\Desktop\github_version_simple\lauetoolsnn\multi_mat_LaueNN\Zr_alpha_ZrO2_mono\model_Zr_alpha_ZrO2_mono.npz", allow_pickle=True)["arr_0"]
     # prediction = np.load(r"C:\Users\purushot\Desktop\github_version_simple\lauetoolsnn\multi_mat_LaueNN\Zr_alpha_ZrO2_mono\model_Zr_alpha_ZrO2_mono.npz", allow_pickle=True)["arr_1"]
 
@@ -374,29 +537,21 @@ if __name__ == "__main__":
     
     # assert np.all(prediction == prediction1)
     
-    # # =============================================================================
-    # #     ### Numpy format
-    # # =============================================================================
-    # wb = read_hdf5(load_weights)
-    # temp_key = list(wb.keys())
+    # max_pred = np.max(prediction, axis = 1)
+    # class_predicted = np.argmax(prediction, axis = 1)
     
-    # # first layer
-    # layer0 = np.dot(x, wb[temp_key[1]]) + wb[temp_key[0]] 
-    # layer0 = np.maximum(0, layer0) ## ReLU activation
-    # # Second layer
-    # layer1 = np.dot(layer0, wb[temp_key[3]]) + wb[temp_key[2]] 
-    # layer1 = np.maximum(0, layer1)
-    # # Third layer
-    # layer2 = np.dot(layer1, wb[temp_key[5]]) + wb[temp_key[4]]
-    # layer2 = np.maximum(0, layer2)
-    # # Output layer
-    # layer3 = np.dot(layer2, wb[temp_key[7]]) + wb[temp_key[6]]
-    # layer3 = softmax(layer3) ## output softmax activation
-
-
-
-
-
+    # # =============================================================================
+    # #     ### Numpy format CNN
+    # # =============================================================================
+    # ##Model weights 
+    # model_weights = read_hdf5(load_weights)
+    # model_key = list(model_weights.keys())
+    # prediction2 = predict_CNN_DNN(codebar, model_weights, model_key)
+    
+    # max_pred1 = np.max(prediction2, axis = 1)
+    # class_predicted1 = np.argmax(prediction2, axis = 1)
+    
+    # assert np.all(class_predicted == class_predicted1)
 
 
 
