@@ -177,25 +177,68 @@ def ApplyExtinctionrules(HKL, Extinc, verbose=0):
         cond1a = K == 0
         cond1b = (L) % 2 != 0
         cond1 = cond1a * cond1b
-
         array_hkl_1 = np.delete(HKL, np.where(cond1 == True)[0], axis=0)
 
         H, K, L = array_hkl_1.T
-
         cond2a = H == 0
         cond2b = L == 0
         cond2c = (K) % 2 != 0
-
         cond2 = cond2a * cond2b * cond2c
         array_hkl_2 = np.delete(array_hkl_1, np.where(cond2 == True)[0], axis=0)
-
+        
+        H, K, L = array_hkl_2.T
         cond3a = H == 0
         cond3b = K == 0
         cond3c = (L) % 2 != 0
         cond3 = cond3a * cond3b * cond3c
-
         array_hkl = np.delete(array_hkl_2, np.where(cond3 == True)[0], axis=0)
-
+    
+    elif Extinc == "VO2_mono2tet":
+        cond4a = L == 0
+        cond4c = (H + K) % 2 != 0
+        cond4 = cond4a * cond4c
+        array_hkl_1 = np.delete(HKL, np.where(cond4 == True)[0], axis=0)
+        
+        H, K, L = array_hkl_1.T
+        cond2a = H == K
+        cond2c = (L) % 2 != 0
+        cond2 = cond2a * cond2c
+        array_hkl_2 = np.delete(array_hkl_1, np.where(cond2 == True)[0], axis=0)
+        
+        H, K, L = array_hkl_2.T
+        cond6a = K == 0
+        cond6b = L == 0
+        cond6c = (H) % 2 != 0
+        cond6 = cond6a * cond6b * cond6c
+        array_hkl_3 = np.delete(array_hkl_2, np.where(cond6 == True)[0], axis=0)
+        
+        H, K, L = array_hkl_3.T
+        cond5a = H == 0
+        cond5b = L == 0
+        cond5c = (K) % 2 != 0
+        cond5 = cond5a * cond5b * cond5c
+        array_hkl_4 = np.delete(array_hkl_3, np.where(cond5 == True)[0], axis=0)
+        
+        H, K, L = array_hkl_4.T
+        cond3a = H == 0
+        cond3b = K == 0
+        cond3c = (L) % 2 != 0
+        cond3 = cond3a * cond3b * cond3c
+        array_hkl = np.delete(array_hkl_4, np.where(cond3 == True)[0], axis=0)
+        
+    elif Extinc == "hcp":
+        cond2a = H == K
+        cond2c = (L) % 2 != 0
+        cond2 = cond2a * cond2c
+        array_hkl_1 = np.delete(HKL, np.where(cond2 == True)[0], axis=0)
+        
+        H, K, L = array_hkl_1.T
+        cond3a = H == 0
+        cond3b = K == 0
+        cond3c = (L) % 2 != 0
+        cond3 = cond3a * cond3b * cond3c
+        array_hkl = np.delete(array_hkl_1, np.where(cond3 == True)[0], axis=0)
+        
     elif Extinc == "VO2_mono2":
 
         cond1 = (H + K) % 2 != 0
