@@ -72,10 +72,10 @@ if __name__ == "__main__":
                     # =============================================================================
                     #       Prediction paarmeters             
                     # =============================================================================
-                    "experimental_directory": "",
-                    "experimental_prefix": "",
-                    "grid_size_x" : 26,           ## Grid X and Y limit to generate the simulated dataset (a rectangular scan region)
-                    "grid_size_y" : 26,  
+                    "experimental_directory": r"C:\Users\purushot\Desktop\Guillou_Laue\scan_0012",
+                    "experimental_prefix": "ech1_map2D_3_",
+                    "grid_size_x" : 1,           ## Grid X and Y limit to generate the simulated dataset (a rectangular scan region)
+                    "grid_size_y" : 5,  
                     "UB_tolerance": [
                                      0.5,
                                      0.5,
@@ -139,7 +139,7 @@ if __name__ == "__main__":
     ## Additional parameters to refine the orientation matrix construction process
     use_om_user = "false"
     nb_spots_consider = 500
-    residues_threshold=0.05
+    residues_threshold=0.5
     nb_spots_global_threshold=8
     option_global = "v2"
     additional_expression = ["none"] # for strain assumptions, like a==b for HCP
@@ -153,10 +153,12 @@ if __name__ == "__main__":
                 continue
             prefix_mat = prefix_mat + "_" + imat
     else:
-        prefix_mat = material_
+        prefix_mat = material_[0]
     
     model_direc = os.getcwd()+"//"+prefix_mat+input_params["prefix"]
     model_weights = model_direc + "//model_"+prefix_mat+".h5"
+    json_file = model_direc + "//model_"+prefix_mat+".json"
+    model_annote = "from_file"
     
     if not os.path.exists(model_direc):
         print("The directory doesn't exists; please veify the path")
@@ -241,7 +243,7 @@ if __name__ == "__main__":
             lim_y,
             tolerance, 
             filenameDirec, 
-            model_direc, 
-            model_weights
+            model_weights,
+            model_annote
             )
 
